package models

import (
	"fmt"
	"regexp"
	"strings"

	"git.paulbsd.com/paulbsd/ipbl/src/config"
)

//var ipv4_regex = `^(((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.|$)){4})/`
var ipv4_cidr_regex = `^(((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.|)){4}\/([1-3])?([0-9])?$)`

// GetTrustlists ...
func GetTrustlists(cfg config.Config) (res []string, err error) {
	var w = Cfg{Key: "trustlist"}
	if exists, _ := cfg.Db.Get(&w); exists {
		res = strings.Split(w.Value, ",")
	}
	return
}

func (wl Trustlist) InsertOrUpdate(cfg config.Config) (err error) {
	var w = Cfg{Key: "trustlist"}
	exists, _ := cfg.Db.Get(&w)
	if exists {
		existing, _ := GetTrustlists(cfg)
		for _, j := range existing {
			if j == wl.IP {
				return fmt.Errorf("ip %s already in config", j)
			}
		}
		existing = append(existing, wl.IP)
		w.Value = strings.Join(existing, ",")
		cfg.Db.ID(w.ID).Update(&w)
	}
	return fmt.Errorf("no trustlist updated")
}

func (wl Trustlist) Delete(cfg config.Config, ip string) (err error) {
	var w = Cfg{Key: "trustlist"}
	exists, _ := cfg.Db.Get(&w)
	var updated []string
	if exists {
		existing, _ := GetTrustlists(cfg)
		for _, sip := range existing {
			if sip != ip {
				updated = append(updated, sip)
			}
		}
		w.Value = strings.Join(updated, ",")
		cfg.Db.ID(w.ID).Update(&w)
	}
	return fmt.Errorf("no trustlist updated")
}

func (wl Trustlist) Verify() bool {
	reg := regexp.MustCompile(ipv4_cidr_regex)
	return reg.MatchString(wl.IP)
}

type Trustlist struct {
	IP string `json:"ip"`
}

// Cfg is ipbl config
type Cfg struct {
	ID    int    `xorm:"pk autoincr" json:"-"`
	Key   string `xorm:"text notnull unique" json:"key"`
	Value string `xorm:"text default" json:"value"`
}
